﻿Option Compare Text

Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.Collections
Imports System.Text.RegularExpressions

Namespace System

   Public Module StringX

      Private _SB As New System.Text.StringBuilder

      ''' <summary> more comfortable than String.Split(separator as String(),count,mode) </summary>
      <Extension()> _
      Public Function Split(ByVal Subj As String, ByVal separator As String, ByVal count As Integer, Optional ByVal mode As StringSplitOptions = StringSplitOptions.None) As String()
         Return Subj.Split(New String() {separator}, count, mode)
      End Function

      ''' <summary> more comfortable to use than String.Split(separator as String(),mode) </summary>
      <Extension()> _
      Public Function Split(ByVal Subj As String, ByVal separator As String, Optional ByVal mode As StringSplitOptions = StringSplitOptions.None) As String()
         Return Subj.Split(New String() {separator}, mode)
      End Function

      ''' <summary> verkettet alles </summary>
      <Extension()> _
      Public Sub [Set](ByRef Subj As String, ByVal ParamArray others() As Object)
         Subj = String.Concat(others)
      End Sub

      ''' <summary> verkettet alles </summary>
      <Extension()> _
      Public Function [And](ByVal Subj As String, ByVal ParamArray others() As Object) As String
         Return String.Concat(Subj, String.Concat(others))
      End Function

      ''' <summary> verkettet alles Strings, fügt Separator dazwischen ein </summary>
      <Extension()> _
      Public Function Join( _
            ByVal Subj As IEnumerable(Of String), _
            Optional ByVal Separator As String = Microsoft.VisualBasic.ControlChars.NewLine) As String
         With New System.Text.StringBuilder
            For Each S In Subj
               .Append(S).Append(Separator)
            Next
            If .Length > 0 Then
               .Remove(.Length - Separator.Length, Separator.Length)
            End If
            Return .ToString
         End With
      End Function

      ''' <returns>den Stringabschnitt ab dem letzten Auftreten von sMatch (ohne sMatch)</returns>
      <Extension()> _
      Public Function LastRightCut(ByVal S$, ByVal sMatch$) As String
         Dim I% = S.LastIndexOf(sMatch)
         If I < 0 Then Return Nothing
         Return S.Substring(I + sMatch.Length)
      End Function

      ''' <returns>den Stringabschnitt ab dem ersten Auftreten von sMatch (ohne sMatch)</returns>
      <Extension()> _
      Public Function RightCut(ByVal S$, ByVal sMatch$) As String
         Dim I% = S.IndexOf(sMatch)
         If I < 0 Then Return Nothing
         Return S.Substring(I + sMatch.Length)
      End Function

      Public Function CreateUnescapeRegex(ByVal Pattern As String) As Regex
         Return New Regex("(?<=(^|[^\\](\\\\)*))" & Pattern) 'der vorangestellte Pattern schließt escaped Matches ('\X' etc.) aus
      End Function

      <Extension()> _
      Public Function HasValue(ByVal Subj As System.String) As Boolean
         Return Not String.IsNullOrEmpty(Subj)
      End Function

      <Extension()> _
      Public Function HasNoValue(ByVal Subj As String) As Boolean
         Return String.IsNullOrEmpty(Subj)
      End Function

      ''' <summary>gibt den String-Abschnitt links des ersten gefundenen Matches zurück</summary>
      <Extension()> _
      Public Function LastLeftCut(ByVal Subj As String, ByVal Pattern As String) As String
         Dim i = Subj.LastIndexOf(Pattern)
         Return If(i >= 0, Subj.Remove(i), Nothing)
      End Function

      ''' <summary>
      ''' gibt den String-Abschnitt links des ersten gefundenen Matches zurück
      ''' </summary>
      <Extension()> _
      Public Function LeftCut(ByVal Subj As String, ByVal Pattern As String) As String
         Dim i = Subj.IndexOf(Pattern)
         Return If(i >= 0, Subj.Remove(i), Nothing)
      End Function

      ' ''Nicht löschen! smart and simple!
      '<Extension()> _
      'Public Function ConcatWith(ByVal Subj As Object, ByVal ParamArray Args() As Object) As String
      '    'Quasi-Umbau von Shared String.Concat() in eine Object-Function
      '    Return Subj.ToString & String.Concat(Args)
      'End Function

      <Extension()> _
      Public Function Between(ByVal Delimiter As Char, ByVal ParamArray Args() As Object) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(ByVal Delimiter As Char, ByVal Args As IEnumerable) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(ByVal Delimiter As String, ByVal ParamArray Args() As Object) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Public Function Between(ByVal Delimiter As String, ByVal Args As IEnumerable) As String
         Return Args.Join(Delimiter)
      End Function

      <Extension()> _
      Private Function Join(ByVal Subj As IEnumerable, ByVal Delimiter As String) As String
         _SB.Clear()
         With Subj.GetEnumerator
            If .MoveNext Then
               _SB.Append(If(.Current, "").ToString)
               While .MoveNext
                  _SB.Append(Delimiter).Append(If(.Current, "").ToString)
               End While
            End If
         End With
         Return _SB.ToString
      End Function

      <Extension()> _
      Public Function IntParsed(ByRef Subj As String) As Integer
         Return Integer.Parse(Subj)
      End Function

      <Extension()> _
      Public Function BoolParsed(ByRef Subj As String) As Boolean
         Return Boolean.Parse(Subj)
      End Function

      <Extension()> _
      Public Function CountChars(ByVal S As String, ByVal Search As Char) As Integer
         Dim I As Integer = S.IndexOf(Search)
         While I >= 0
            CountChars += 1
            I = S.IndexOf(Search, I + 1)
         End While
      End Function

      <Extension()> _
      Public Function CountSubStrings( _
            ByVal S As String, _
            ByVal Search As String, _
            Optional ByVal Comparison As StringComparison = StringComparison.CurrentCulture) As Integer
         Dim I As Integer = S.IndexOf(Search, 0, Comparison)
         While I >= 0
            CountSubStrings += 1
            I = S.IndexOf(Search, I + 1, Comparison)
         End While
      End Function

   End Module


End Namespace